use std::{
	io::{self, Write},
	path::PathBuf,
	str::FromStr,
};

use rusqlite::{Connection, params_from_iter};

use crate::{
	DB_PATH,
	cli::{self, FilesArgs, TagArgs, TagsArgs},
	try_database,
};

pub fn handle_tag(args: TagArgs) {
	try_database();

	let mut conn = Connection::open(DB_PATH).unwrap();

	tag_file(&mut conn, args.file, args.tags);
}

pub fn handle_tags(args: TagsArgs) {
	try_database();

	let conn = Connection::open(DB_PATH).unwrap();

	match args.commands {
		Some(cli::TagsCommands::List) | None => {
			let mut w = io::stdout();
			let tags = list_tags(&conn);
			for tag in tags {
				writeln!(&mut w, "{}", tag).unwrap();
			}
			w.flush().unwrap();
		}
		Some(cli::TagsCommands::Add { add }) => add_tags(&conn, add),
		Some(cli::TagsCommands::Remove { remove }) => remove_tags(&conn, remove),
	};
}

fn list_tags(conn: &Connection) -> Vec<String> {
	let mut stmt = conn.prepare("SELECT name FROM tag").unwrap();
	let result = stmt.query_map([], |row| row.get(0)).unwrap();

	let mut tags = Vec::new();
	for name in result {
		tags.push(name.unwrap());
	}

	tags
}

fn remove_tags(conn: &Connection, tags: Vec<String>) {
	let mut query = r#"DELETE FROM tag WHERE name IN ("#.to_string();

	for (i, _tag) in tags.iter().enumerate() {
		query.push('?');
		if i < tags.len() - 1 {
			query.push(',');
		}
	}

	query.push(')');

	conn.execute(&query, params_from_iter(tags)).unwrap();
}

fn add_tags(conn: &Connection, tags: Vec<String>) {
	let mut query = r#"INSERT INTO tag(name) VALUES"#.to_string();

	for (i, _tag) in tags.iter().enumerate() {
		query.push_str("(?)");
		if i < tags.len() - 1 {
			query.push(',');
		}
	}

	conn.execute(&query, params_from_iter(tags)).unwrap();
}

fn tag_file(conn: &mut Connection, file: PathBuf, tags: Vec<String>) {
	let file = file.to_str().unwrap().to_string();

	let tx = conn.transaction().unwrap();
	{
		let mut stmt = tx
			.prepare(r#"INSERT INTO file(path) VALUES (?) RETURNING id"#)
			.unwrap();
		let file_id = stmt
			.query_row([file], |row| -> Result<i32, rusqlite::Error> { row.get(0) })
			.unwrap();

		let mut sql = "SELECT id FROM tag WHERE name IN ".to_string();
		sql.push('(');
		let size = tags.len();
		for (i, _tag) in tags.iter().enumerate() {
			sql.push('?');
			if i < size - 1 {
				sql.push(',');
			}
		}
		sql.push(')');

		let mut stmt = tx.prepare(&sql).unwrap();
		let result = stmt
			.query_map(
				params_from_iter(tags),
				|row| -> Result<i32, rusqlite::Error> { row.get(0) },
			)
			.unwrap();

		let mut tags_ids = Vec::new();
		for id in result.flatten() {
			tags_ids.push(id);
		}

		let mut stmt = tx
			.prepare(r#"INSERT INTO file_tag(tag_id, file_id) VALUES (?, ?)"#)
			.unwrap();

		for tag_id in tags_ids {
			stmt.execute((file_id, tag_id)).unwrap();
		}
	}
	tx.commit().unwrap();
}

pub fn handle_files(args: FilesArgs) {
	if let Some(query) = args.query {
		// match PathBuf::from_str(&query) {
		// 	Ok(path) => {
		// 		dbg!(path);
		// 	}
		// 	Err(_) => {
		// 		dbg!("a query");
		// 	}
		// }
	}
}
